/*
 *  Cube.h
 *
 *  Cube class for Cube Polygonization Algorithm
 *
 *  Copyright (C) 2002 Tobias Dyckerhoff
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#ifndef CUBE_H
#define CUBE_H

#include "lib/permutations.h"
#include <vector>
#include <list>

/**Cube render helper class.
   Class for a cube in the rasterized 3-dim space which is used
   to interpolate the approximated surface of the hyperspace.
   This is done by use of permutations that operate on the cube's
   corner vertices. */
class Cube{
  
  unsigned char inbetw( unsigned char i, unsigned char j); 
  static list<Perm> rot; //list with the permutation group of the cube
  static vector<vector<double> > edge_vec; // vectors pointing to the middle point of edges

 public:

  /** Constructor */
  Cube(int i);

  /** Alternative Constructor setting up origin vector */
  Cube(int i, double a, double b, double c);

  /** Count how many vertices are prisoners */
  int in_count;

  /** Saves the vertex at the origin of the cube */
  vector<double> v;

  /** In cache */
  vector<bool> in;

  /** Saves normal vector of approximated plane*/
  vector<double> norm;

  /** The approximated polygon */
  list<vector<double> > pol;

  /** Polygonize the cube using cache*/
  void polygonize(list<unsigned char >* poly_list, vector<double>* norm);

  /** Returns a list containing indices pointing to the edge vectors
      that define the approximated polygon. It
      is used for creating the cache */
  list<unsigned char>* get_poly_index_list();

  /** Calculates the normal for a given polygon defined by poly_list*/
  vector<double>* get_normal(list<unsigned char >* poly_list);

  /** Initialize the rotation group of the cube */
  static void init_rot();

  /** Initialize the edge_vec using the given edge_length */
  static void init_edge_vec(double edge_length);
  
};


#endif
