/*
 *  GenericViewer.cc
 *
 *  Generic Viewer class derived from Viewer
 *
 *  Copyright (C) 2002 Tobias Dyckerhoff
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#include "GenericViewer.h"


GenericViewer::GenericViewer(char* label):  Viewer(label)
{

  /* this button is toggled first in the constructor of caminfb!!!*/
  mouse_mode=2;


  /* connecting all the signals. */

  Viewer::gl_changed_event.connect(slot(this,&GenericViewer::update_caminfb));

  caminfb.camera_Gtk_CoordEntry.entry_activated.connect(slot(this,&GenericViewer::entry_activated));
  caminfb.lookat_Gtk_CoordEntry.entry_activated.connect(slot(this,&GenericViewer::entry_activated));
  caminfb.up_Gtk_CoordEntry.entry_activated.connect(slot(this,&GenericViewer::entry_activated));
  caminfb.light_Gtk_CoordEntry.entry_activated.connect(slot(this,&GenericViewer::entry_activated));

  caminfb.mode_set.connect(slot(this,&GenericViewer::set_mouse_mode));

  camera.pos_update.connect(slot(&caminfb.camera_Gtk_CoordEntry,&CoordEntry::set_coord));
  camera.lookat_update.connect(slot(&caminfb.lookat_Gtk_CoordEntry,&CoordEntry::set_coord));
  camera.up_update.connect(slot(&caminfb.up_Gtk_CoordEntry,&CoordEntry::set_coord));

 inftools.tool_event.connect(slot(this,&GenericViewer::tool_handler));

  /* the CameraControl window will be always in front of the GenericViewer 
     window */
 caminfb.set_transient_for(*this);


 /* setting up the toolbar */

 using namespace Gtk::Toolbar_Helpers;
 Viewer::upper_hbox.pack_start(toolbar);
 toolbar.tools().push_back(Space());
 toolbar.tools().push_back(Space()); 
 toolbar.tools().push_back(Element(inftools));


 /* setting up the CoordEntries of the CameraControl window */

 caminfb.camera_Gtk_CoordEntry.set_coord(camera.pos);
 caminfb.lookat_Gtk_CoordEntry.set_coord(camera.lookat);
 caminfb.up_Gtk_CoordEntry.set_coord(camera.up);
 caminfb.light_Gtk_CoordEntry.set_coord(gl_light_pos);

}


 
gint GenericViewer::gl_motion_notify_handler_imp(GdkEventMotion* event)
{
  /* old mode so the Viewer methods are called */ 
  if (mouse_mode==2) Viewer::gl_motion_notify_handler_imp(event);

  /* new mode */  
  if (mouse_mode==1) 
    {

      gint x,y;
      
      GdkModifierType state;
      
      if (event->is_hint)
	gdk_window_get_pointer (event->window, &x, &y, &state);
      else
	{
	  x = (gint) event->x;
	  y = (gint) event->y;
	  state =(GdkModifierType) event->state;
	}
      
      if ((state & GDK_BUTTON1_MASK)&&mouse1_pressed)
	{
	  Vector3 look=Viewer::camera.lookat-Viewer::camera.pos;
	  look=look/look.abs();
	  Vector3 up=Viewer::camera.up;
	  Vector3 side=up^look;
	  Vector3 change=up*(mouse_start_y-y)/100
	                 +side*(mouse_start_x-x)/100;
	  mouse_start_y=y;
	  mouse_start_x=x;

	  Viewer::camera.lookat=Viewer::camera.lookat+change;
	  Viewer::camera.pos=Viewer::camera.pos+change;
 	 
	  Viewer::updategl();
	  Viewer::drawgl();


	  caminfb.camera_Gtk_CoordEntry.set_coord(camera.pos);
	  caminfb.lookat_Gtk_CoordEntry.set_coord(camera.lookat);
	  caminfb.up_Gtk_CoordEntry.set_coord(camera.up);

	}
      if ((state & GDK_BUTTON2_MASK)&&mouse2_pressed)
	{
	   Vector3 look=Viewer::camera.lookat-Viewer::camera.pos;
	  look=look/look.abs();
	  
	  Vector3 change=look*(mouse_start_y-y)/100;
	  mouse_start_y=y;
	  mouse_start_x=x;
	 
	  if (mouse_start_y-y>0)
	    {
	      Viewer::camera.lookat=Viewer::camera.lookat+change;
	      Viewer::camera.pos=Viewer::camera.pos+change;
	    }
	  else
	    {
	      Viewer::camera.pos=Viewer::camera.pos+change;
	      Viewer::camera.lookat=Viewer::camera.lookat+change;
	    }


	  Viewer::updategl();
	  Viewer::drawgl();

	  
	  caminfb.camera_Gtk_CoordEntry.set_coord(camera.pos);
	  caminfb.lookat_Gtk_CoordEntry.set_coord(camera.lookat);
	  caminfb.up_Gtk_CoordEntry.set_coord(camera.up);
	}
      
      
      if ((state & GDK_BUTTON3_MASK)&&mouse3_pressed)
	{


	}
      
    }
}

gint GenericViewer::gl_button_press_handler_imp(GdkEventButton* event)
{
  /* old mode so the Viewer methods are called */ 
  if (mouse_mode==2) Viewer::gl_button_press_handler_imp(event);

  /* new mode */    
  if(mouse_mode==1){
    if (event->button==1){
      mouse_start_x=event->x;
      mouse_start_y=event->y;
      mouse1_pressed=1;
    }
    if (event->button==2){
      mouse_start_x=event->x;
      mouse_start_y=event->y;
      mouse2_pressed=1;
    }
    
    if (event->button==3){
      mouse_start_x=event->x;
      mouse_start_y=event->y;
      mouse3_pressed=1;
  }
  }
}

gint GenericViewer::gl_button_release_handler_imp(GdkEventButton* event)
{ 
  /* old mode so the Viewer methods are called */ 
  if (mouse_mode==2) Viewer::gl_button_release_handler_imp(event);

  /* new mode */  
  if (mouse_mode==1)
    {
      if (event->button==1)  
	{
	  mouse1_pressed=0;
	}
      
      if (event->button==2)  
	{
	  mouse2_pressed=0;
	}
      
      if (event->button==3)  
	{
	  mouse3_pressed=0;
	}
      
    }
}




void GenericViewer::set_mouse_mode(int mode)
{
  mouse_mode=mode;
}



void GenericViewer::update_caminfb()
{
  caminfb.camera_Gtk_CoordEntry.set_coord(camera.pos);
  caminfb.lookat_Gtk_CoordEntry.set_coord(camera.lookat);
  caminfb.up_Gtk_CoordEntry.set_coord(camera.up);
  caminfb.light_Gtk_CoordEntry.set_coord(gl_light_pos);
}



void GenericViewer::tool_handler(int id)
{
  /* the CameraControl window will be showed/hidden when its button is pressed */
  if (id==1) 
    { 

      if(caminfb.active==0)
	{
	  caminfb.active=1;
	  caminfb.show_all();
	}
      else
	{
	  caminfb.active=0;
	  caminfb.hide_all();
	}
    }

}



void GenericViewer::entry_activated(int id){

  //need to update up first as it might be changed later due to orthogonalization
  //   if (id==caminfb.up_Gtk_CoordEntry.get_id())
  Viewer::camera.set_up(caminfb.up_Gtk_CoordEntry.get_coord());
     //  if (id==caminfb.camera_Gtk_CoordEntry.get_id())
  Viewer::camera.set_pos(caminfb.camera_Gtk_CoordEntry.get_coord());
   
     //  if (id==caminfb.lookat_Gtk_CoordEntry.get_id())
  Viewer::camera.set_lookat(caminfb.lookat_Gtk_CoordEntry.get_coord());
   
     //  if (id==caminfb.light_Gtk_CoordEntry.get_id())
  Viewer::set_light(caminfb.light_Gtk_CoordEntry.get_coord());
  
  updategl();
  drawgl();
}
