/*
 *  MouseControl.h
 *
 *  Helper for mouse control in Viewer
 *
 *  Copyright (C) 2002 Tobias Dyckerhoff
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#ifndef MOUSECONTROL_H
#define MOUSECONTROL_H


#include <gtk--/box.h> //just to declare gint and gboolean


/** This class contains some members that are frequently needed when a widget has
    a own mouse control. It can ask for mouse focus with a signal and handle
    mouse events, which should be channeled through by the managing object (the 
    QuatFractViewer in our case). 
*/
class MouseControl
{
 public:
  /** Constructor */
  MouseControl();

  /** Virtual methods analogous to the standard mouse event handlers */
  virtual gint mouse_motion_imp(GdkEventMotion* event);

  /** Virtual methods analogous to the standard mouse event handlers */
  virtual gint mouse_press_imp(GdkEventButton* event);

  /** Virtual methods analogous to the standard mouse event handlers */
  virtual gint mouse_release_imp(GdkEventButton* event);
  
  /** Signal to ask the managing objects for control over the mouse. T
      he boolean tells if focus is to be set or released */
  SigC::Signal1<void, gboolean> mouse_focus_request;

  /** variables that can be used for the mouse control*/
  gdouble mouse_start_x; 
  /** variables that can be used for the mouse control*/
  gdouble  mouse_start_y;
  /** variables that can be used for the mouse control*/
  gboolean mouse1_pressed;
  /** variables that can be used for the mouse control*/
  gboolean mouse2_pressed;
  /** variables that can be used for the mouse control*/
  gboolean mouse3_pressed;
};

#endif
