/*
 *  QuatFractCreator_Tetra.cc
 *
 *  Tetra Polygonization Algorithm
 *
 *  Copyright (C) 2002 Tobias Dyckerhoff
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#include "QuatFractCreator.h"
#include "QuatMath.h"
#include "OpenGLUtils.h"
#include "Cube.h"
#include "tetra_Cube.h"


#include <vector>
#include <list>
#include <map>
#include <string>

//shall be removed (capsulating from opengl)

#include <GL/gl.h> 
#include <GL/glu.h> 
#include <GL/glut.h> 


/* this is the brute force method: just iterate through 
   all cells and check if we are on the surface or not
   , if we are render a polygon */

void QuatFractCreator::tetra_polygonize() {
  double min = -1;
  double max = 1;
  double step = 0.1;
  unsigned int pol_i = 0;

  /* here we'll save all surface polygons
     will be converted into a vector later */
  list<list<vector<double> > > polys;
  
  /* map the surface vertices to the list of 
     the (four) polygons it is involved in */
  map< vector<double>, list<int> > point_to_polys;

  /* map the surface vertices to their normals 
     , this saved redundant calculations */
  map< vector<double>, vector<double> > point_to_normal;

  /* Quat an vector buffer for iteration */
  Quat *x = new Quat();
  vector<double> vect(3);

  /* draw the Cube surrounding the iteration zone */
  glColor3f(0, 1.0, 0);
  glutWireCube (2*max);

  /* draw coordinate axes */
  glBegin(GL_LINES);
    glColor3f(1, 0, 0);
    glVertex3f( 0, 0, 0);
    glVertex3f( 3, 0, 0);
    glVertex3f( 0, 0, 0);
    glVertex3f( 0, 3, 0);
    glVertex3f( 0, 0, 0);
    glVertex3f( 0, 0, 3);
  glEnd();
  output(3, 0, 0, "x");
  output(0, 3, 0, "y");
  output(0, 0, 3, "z");
  

  /* the cut into 4-dim quat-space is done by brutely setting
     x4 = 0, this is the most efficient and least general way
     will have to find a fast algo doing that with every
     hyperplane */
  for( double a = 2*min; a < 2*max; a += step) {
    for( double b = min; b < max; b += step) {
      for( double c = min; c < max; c += step){
	/* a cube is created and the vertices are iterated */
	int r = 0;
	TetraCube cube(8);
	for( int i = 0; i < 2; i++) 
	  for( int j = 0; j < 2; j++) 
	    for( int k = 0; k < 2; k++) {
	      vect[0] = a+i*step;
	      vect[1] = b+j*step;
	      vect[2] = c+k*step;
	      cube.v[r] = vect;
	      *x = Quat(vect[0], vect[1], vect[2], 0);
	      if( konv(*x) ) {
		cube.in_count++;
		cube.in[r] = true;
	      } else {
		cube.in[r] = false;
	      }
	      r++;
	    }
	
	/* if we are at the surface we save the cube in the
	   polys list and add it to the map for each vertex 
	   involved */
 	if( cube.in_count > 0 && cube.in_count < 8) {
	  list<list<vector<double> > > polygons = cube.polygonize();
	  for( list<list<vector<double> > >::iterator pl = polygons.begin(); 
	       pl != polygons.end(); pl++) {
	    polys.push_back( *pl);
	    for( list<vector<double> >::iterator pv = pl->begin(); 
	       pv != pl->end(); pv++) 
	      point_to_polys[*pv].push_back(pol_i);
	    pol_i++;
	  }
	}
      }
    }
    cout << ".";
    cout.flush();
  }
  
  cout << "polygon count: " << pol_i << endl;
  cout.flush();

  /* copy the list to a vector in order to easy indexing */
  vector<list<vector<double> > > pol_v(polys.begin(), polys.end());


    int debug_c = 0;   // just print a few polys for debugging reasons
    int limit = 20;
  /* finally iterate through polygons, merge normals at each vertex
     and draw the surface */
  for( vector<list<vector<double> > >::iterator pi = pol_v.begin(); pi != pol_v.end(); pi++) {

    if(debug_c > limit)
            break;
    
    
    glColor3f(1, 1, 1);
    if(pi->size() == 4)
      glBegin(GL_TRIANGLES);
    else
      glBegin(GL_QUADS);
    //    glBegin(GL_POLYGON);

    /* iterate through vertices in polygon *pi */
//     vector<double> n_backup = pi->back();  // backup normal
//     pi->pop_back();
    int v_count = 1;
    for( list< vector<double> >::iterator vi = pi->begin(); 
	 vi != pi->end(); vi++) {

      if( ++v_count > pi->size())
	break;

      vector< double> normal(3);
      
      /* is *vi already cached? */
      if( point_to_normal[*vi].size() == 0) {
	for(int i = 0; i < 3; i++)
	  normal[i] = 0;
	
	/* find all polygons including vertex *vi and 
	   merge normals */
	list< int> l = point_to_polys[*vi];
	for( list< int>::iterator pi = l.begin(); pi != l.end(); pi++) {
	  for(int i = 0; i < 3; i++)
	    normal[i] += (pol_v[*pi].back())[i];
	}
	double nabs = sqrt(normal[0]*normal[0]
			   +normal[1]*normal[1]
			   +normal[2]*normal[2]);
	
	if( nabs == 0)
	  cerr << "aaaaaaaaaaaaaaaaaahh no normal" << endl;

	for(int i = 0; i < 3; i++)
	  normal[i] /= nabs;
	
	/* we will map *vi its normal in order to eliminate 
	   unnecessary draw operations */
	point_to_normal[*vi] = normal;
	
      } 
      /* *vi is cached, so we'll take the cached normal */
      else {
	normal = point_to_normal[*vi];
      }
      
      
      /* now we have the combined normal and are able to 
	 set the vertex */
      glNormal3f( normal[0], normal[1], normal[2]);
      glVertex3f( (*vi)[0], (*vi)[1], (*vi)[2]);
      

    }
    glEnd();
    //    pi->push_back(n_backup); // restore normal
  }

  debug_c = 0;

    /* show normals */
//   for( vector<list<vector<double> > >::iterator pi = pol_v.begin(); pi != pol_v.end(); pi++) {

//     if(debug_c > limit)
//       break;
    
//     int v_count = 1;
//     for( list< vector<double> >::iterator vi = pi->begin(); 
// 	 vi != pi->end(); vi++) {

//       if( ++v_count > pi->size())
// 	break;
      
//        vector<double> no = point_to_normal[*vi];

//        if( no.size() != 0) {
// 	 glColor3f( 1, 0, 0); 
//  	 glBegin(GL_LINES); 
//             glVertex3f((*vi)[0], (*vi)[1], (*vi)[2]); 
//             glVertex3f((*vi)[0]+no[0]/20, (*vi)[1]+no[1]/20, (*vi)[2]+no[2]/20); 
//  	 glEnd(); 
//        }

//     }
//   } 


}



/* lil test if the polygonization works correct */

void QuatFractCreator::tetra_test() {
  double min = -1;
  double max = 1;
  double step = 1;

  unsigned int pol_i = 0;

  /* here we'll save all surface polygons
     will be converted into a vector later */
  list<list<vector<double> > > polys;
  
  /* map the surface vertices to the list of 
     the (four) polygons it is involved in */
  map< vector<double>, list<int> > point_to_polys;

  /* map the surface vertices to their normals 
     , this saved redundant calculations */
  map< vector<double>, vector<double> > point_to_normal;


  Quat *x = new Quat();
  vector<double> vect(3);
  vector<bool> test(8);
  for( int i = 0; i < 7; i++) 
    test[i] = 0;


  /* this is the area in which we are rendering */
  glColor3f(0, 1.0, 0);
  glutWireCube (2*max);

  glColor3f(1.0, 1.0, 1.0);
  
  TetraCube surface_cube(8);

  /* randomly pick cells */

  int switch_c = 0;
  for(  double b = 1 ; b > -2; b-=step) { 
  for(  double a = -1; a < 1; a+=step) { 
  for(  double c = -1; c < 1; c+=step) { 
    switch(switch_c++) {
    case 0:
      test[0] = 0;
      test[1] = 0;
      test[2] = 0; 
      test[3] = 0; 
      test[4] = 0; 
      test[5] = 1; 
      test[6] = 0; 
      test[7] = 0; 
      break;
    case 1:
      test[0] = 0;
      test[1] = 0;
      test[2] = 0; 
      test[3] = 0; 
      test[4] = 1; 
      test[5] = 0; 
      test[6] = 0; 
      test[7] = 0; 
      break;
    case 2:
      test[0] = 0;
      test[1] = 1;
      test[2] = 0; 
      test[3] = 0; 
      test[4] = 0; 
      test[5] = 0; 
      test[6] = 0; 
      test[7] = 0; 
      break;
    case 3:
      test[0] = 1;
      test[1] = 0;
      test[2] = 0; 
      test[3] = 0; 
      test[4] = 0; 
      test[5] = 0; 
      test[6] = 0; 
      test[7] = 0; 
      break;

    case 4:
      test[0] = 0;
      test[1] = 0;
      test[2] = 0; 
      test[3] = 0; 
      test[4] = 0; 
      test[5] = 1; 
      test[6] = 0; 
      test[7] = 1; 
      break;
    case 5:
      test[0] = 0;
      test[1] = 0;
      test[2] = 0; 
      test[3] = 0; 
      test[4] = 1; 
      test[5] = 0; 
      test[6] = 1; 
      test[7] = 0; 
      break;
    case 6:
      test[0] = 0;
      test[1] = 1;
      test[2] = 0; 
      test[3] = 1; 
      test[4] = 0; 
      test[5] = 0; 
      test[6] = 0; 
      test[7] = 0; 
      break;
    case 7:
      test[0] = 1;
      test[1] = 0;
      test[2] = 1; 
      test[3] = 0; 
      test[4] = 0; 
      test[5] = 0; 
      test[6] = 0; 
      test[7] = 0; 
      break;

    case 8:
      test[0] = 0;
      test[1] = 0;
      test[2] = 0; 
      test[3] = 0; 
      test[4] = 0; 
      test[5] = 0; 
      test[6] = 0; 
      test[7] = 1; 
      break;
    case 9:
      test[0] = 0;
      test[1] = 0;
      test[2] = 0; 
      test[3] = 0; 
      test[4] = 0; 
      test[5] = 0; 
      test[6] = 1; 
      test[7] = 0; 
      break;
    case 10:
      test[0] = 0;
      test[1] = 0;
      test[2] = 0; 
      test[3] = 1; 
      test[4] = 0; 
      test[5] = 0; 
      test[6] = 0; 
      test[7] = 0; 
      break;
    case 11:
      test[0] = 0;
      test[1] = 0;
      test[2] = 1; 
      test[3] = 0; 
      test[4] = 0; 
      test[5] = 0; 
      test[6] = 0; 
      test[7] = 0; 
      break;
    }

  int r = 0;
  TetraCube cube(8);
  for( int i = 0; i < 2; i++) 
    for( int j = 0; j < 2; j++) 
      for( int k = 0; k < 2; k++) {
	vect[0] = a+i*step;
	vect[1] = b+j*step;
	vect[2] = c+k*step;
	cube.v[r] = vect;
	cout << vect[0] << "," << vect[1] << "," << vect[2] << endl;

	if( test[r]) {
	  cube.in_count++;
	  cube.in[r] = true;
	} else {
	  cube.in[r] = false;
	}
	r++;
      }

  /* if we are at the surface we save the cube in the
     polys list and add it to the map for each vertex 
     involved */
  if( cube.in_count > 0 && cube.in_count < 8) {
    list<list<vector<double> > > polygons = cube.polygonize();
    for( list<list<vector<double> > >::iterator pl = polygons.begin(); 
	 pl != polygons.end(); pl++) {
      polys.push_back( *pl);
      for( list<vector<double> >::iterator pv = pl->begin(); 
	   pv != pl->end(); pv++) 
	point_to_polys[*pv].push_back(pol_i);
      pol_i++;
    }
  }
  }
  }}
  cout << "polygon count: " << pol_i << endl;
  cout.flush();

  /* copy the list to a vector in order to easy indexing */
  vector<list<vector<double> > > pol_v(polys.begin(), polys.end());


   // int debug_c = 0;   // just print a few polys for debugging reasons

  /* finally iterate through polygons, merge normals at each vertex
     and draw the surface */
  for( vector<list<vector<double> > >::iterator pi = pol_v.begin(); pi != pol_v.end(); pi++) {

     //if(debug_c++ > 2)
       //    break;
    
    
    glColor3f(1, 1, 1);
    if(pi->size() == 4)
      glBegin(GL_TRIANGLES);
    else
      glBegin(GL_QUADS);


    /* iterate through vertices in polygon *pi */
//     vector<double> n_backup = pi->back();  // backup normal
//     pi->pop_back();
    int v_count = 1;
    cout << pi->size() << endl;
    for( list< vector<double> >::iterator vi = pi->begin(); 
	 vi != pi->end(); vi++) {

      if( ++v_count > pi->size())
	break;

      vector< double> normal(3);
      
      /* is *vi already cached? */
      if( point_to_normal[*vi].size() == 0) {
	for(int i = 0; i < 3; i++)
	  normal[i] = 0;
	
	/* find all polygons including vertex *vi and 
	   merge normals */
	list< int> l = point_to_polys[*vi];
	for( list< int>::iterator pi = l.begin(); pi != l.end(); pi++) {
	  for(int i = 0; i < 3; i++)
	    normal[i] += (pol_v[*pi].back())[i];
	}
	double nabs = sqrt(normal[0]*normal[0]
			   +normal[1]*normal[1]
			   +normal[2]*normal[2]);
	
	for(int i = 0; i < 3; i++)
	  normal[i] /= nabs;
	
	/* we will map *vi its normal in order to eliminate 
	   unnecessary draw operations */
	point_to_normal[*vi] = normal;
	
      } 
      /* *vi is cached, so we'll take the cached normal */
      else {
	normal = point_to_normal[*vi];
      }
      
      
      /* now we have the combined normal and are able to 
	 set the vertex */
      glNormal3f( normal[0], normal[1], normal[2]);
      glVertex3f( (*vi)[0], (*vi)[1], (*vi)[2]);
      cout <<  (*vi)[0]<< ","<< (*vi)[1]<<"," <<(*vi)[2] << endl;
      cout <<  "n="<<normal[0]<<","<< normal[1]<<","<< normal[2]<< endl;

    }
    glEnd();
    cout << "--"  << endl;

    //    pi->push_back(n_backup); // restore normal
  }

  
    /* show normals */
  for( vector<list<vector<double> > >::iterator pi = pol_v.begin(); pi != pol_v.end(); pi++) {

    
    int v_count = 1;
    for( list< vector<double> >::iterator vi = pi->begin(); 
	 vi != pi->end(); vi++) {

      if( ++v_count > pi->size())
	break;
      
       vector<double> no = point_to_normal[*vi];

       if( no.size() != 0) {
	 glColor3f( 1, 0, 0); 
 	 glBegin(GL_LINES); 
            glVertex3f((*vi)[0], (*vi)[1], (*vi)[2]); 
            glVertex3f((*vi)[0]+no[0]/20, (*vi)[1]+no[1]/20, (*vi)[2]+no[2]/20); 
 	 glEnd(); 
       }

    }
  } 
  
}

