/*
 *  QuatFractViewer.cc
 *
 *  Fractal Viewer derived from Generic Viewer
 *
 *  Copyright (C) 2002 Tobias Dyckerhoff
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#include "QuatFractViewer.h"

#include <stdio.h>
#include <gdk/gdk.h>
#include <gtk/gtk.h>

#include <gtk--/main.h>
#include <gtk--/toolbar.h>
#include <gtk--/button.h>
#include <gtk--/togglebutton.h>
#include <gtk--/radiobutton.h>
#include <gtk--/box.h>
#include <gtk--/window.h>


#define THREADING 0

using SigC::bind;
using SigC::slot;

bool QuatFractViewer::finish_render = true;


/* static default qf ? */
QuatFractViewer::QuatFractViewer():
  GenericViewer("Fractal Viewer"), 
  qf( -1, -0.13, 0.0, 0.0)
{
  
  /* this is hardcoded just for testing reasons */
  char* parfile = "povray/defines.tpl";
  char* pov_tplfile = "povray/ani.pov";
  char* ini_tplfile = "povray/quat.ini";


  /* read some template mapping file */
  string::size_type i, j;  
  ifstream from( parfile);
  if( !from) 
    cerr << "error cannot open " << parfile << endl;

  string tpl;
  map<string, string*> lookup;

  // parse in templates
  while( getline(from, tpl)) {
    i = tpl.find('=');
    if( i != string::npos)
      lookup[string(tpl.begin(), tpl.begin()+i)] = 
	new string( tpl.begin()+i+1, tpl.end());
  }


  po = manage( new PovrayOut( lookup, pov_tplfile, ini_tplfile, pathmaster.get_path()));


  mouse_focus=0;

  gfp = manage( new GtkFractParams( qf.get_fparams()));
  gfp->update_request.connect( slot( this, &QuatFractViewer::update_params));
  gfp->redraw.connect(slot(this, &QuatFractViewer::drawgl));
  gfp->mouse_focus_request.connect(bind<int>(slot(this, &QuatFractViewer::mouse_manager),4));
  gfp->camera_stats_input.connect(slot(&camera, &SpaceEntity::get_stats));

  gfp->set_transient_for(*this);

  pathmaster.camera_stats_input.connect(slot(&camera, &SpaceEntity::get_stats));
  pathmaster.redraw.connect(slot(this, &QuatFractViewer::drawgl));
  pathmaster.path_update.connect(slot(this, &QuatFractViewer::update_path));

  using namespace Gtk::Toolbar_Helpers;
  frtools.tool_event.connect(slot(this,&QuatFractViewer::tool_handler));
  GenericViewer::toolbar.tools().push_back(Space());
  GenericViewer::toolbar.tools().push_back(Element(frtools));
  GenericViewer::show_all();
  //(-0.2,0.8,0,0); //(-0.218,-0.113,-0.181,-0.496);//(-0.2,0.8,0,0); //(-1, 0.6, 0, 0);

  /* choose rendering method */
  qf.set_method(CUBE);
  GenericViewer::initgl();  

  po->hide();
  gfp->hide();
  
}



void QuatFractViewer::mouse_manager(gboolean on, int id)
{
  switch(id){
    //4 is gfp
  case 4:
    if (on) mouse_focus=4;
    else mouse_focus=0;
    break;
  }
}



void QuatFractViewer::tool_handler(int id) {
  switch(id) {
  case 1: 
    pathmaster.activate();
    break;
  case 2:
    if( frtools.get_toggle(2))
      po->show_all();
    else 
      po->hide_all();
    break;
  case 3:
    render_fractal();
    break;
  case 4:
    if( frtools.get_toggle(4))
      gfp->show_all();
    else 
      gfp->hide_all();
    break;

  }
}


void QuatFractViewer::update_params() {
  qf.set_fparams( *gfp->get_params());

  glNewList(3, GL_COMPILE);
  draw_boundingbox();
  glEndList();

}


void QuatFractViewer::update_path() {
  po->update_path(pathmaster.get_path());
  po->update_endtime( pathmaster.get_path().back()->time);
}


gint QuatFractViewer::init_gl_context_imp(){

  /* call parent gl init */
  GenericViewer::init_gl_context_imp();

  /* this is how we like the opengl scene */
  
//   GLfloat light_ambient[] = { 0.1, 0.1, 0.1, 1.0 };
//   GLfloat light_diffuse[] = { 1, 1, 1, 1.0 };
//   GLfloat light_specular[] = { 0, 0, 0, 1.0 };
  
//   glLightfv(GL_LIGHT0, GL_AMBIENT, light_ambient);
//   glLightfv(GL_LIGHT0, GL_DIFFUSE, light_diffuse);
//   glLightfv(GL_LIGHT0, GL_SPECULAR, light_specular);
  
  GLfloat mat_specular[] = { 0., 0., 0., 1.0 };
  GLfloat mat_diffuse[] = { 0.3, 0.3, 0.3, 1.0 };
  
  glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, mat_specular);
  glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, mat_diffuse);

  glColorMaterial(GL_FRONT_AND_BACK, GL_DIFFUSE);
  glEnable(GL_COLOR_MATERIAL);  
  
  glEnable(GL_LIGHTING);
//   glEnable(GL_LIGHT0);
  
  glEnable(GL_DEPTH_TEST);
  glShadeModel(GL_SMOOTH);    
  

  /* initialize cube algorithm cache */
  qf.cube_init();

  glNewList(2, GL_COMPILE);
  draw_coord();
  glEndList();

  glNewList(3, GL_COMPILE);
  draw_boundingbox();
  glEndList();


  return 0;
}


void QuatFractViewer::render_fractal() {

  int t_id;
  pthread_t pt;

  /* render the display list(s) */
  cout << "rendering->" << endl;
  cout.flush();


  switch(qf.get_method()) {
  case CUBE:
#if THREADING
    t_id = pthread_create(&pt, NULL, enter_thread, (void*) &qf);
#else
  qf.init_conv_map();
  qf.cube_polygonize();
  QuatFractViewer::finish_render = false;
  glNewList(1, GL_COMPILE);
    qf.cube_merge_n_draw();
  glEndList();
  QuatFractViewer::finish_render = true;
#endif
    break;
  case TETRA:
    cout << "uncomment me" << endl;
    //qf.tetra_polygonize();
    break;
  case COMBI:
    cout << "implement me" << endl;
    //qf.combi_polygonize();
    break;
  case TRACK:
    cout << "implement me" << endl;
    //qf.track_polygonize();
    break;
  }
  drawgl();
}


gint QuatFractViewer::draw_gl_context_imp(){

  /* draw coordinate axes */
  glCallList(2);
  glCallList(3);

//   /* draw the Cube surrounding the iteration zone */
//   glColor3f(0, 1.0, 0);
//   glutWireCube (2*fparams.zMax);

  

  /* call the list to render */
  if( QuatFractViewer::finish_render)
    glCallList(1);

  pathmaster.draw();

  glPushMatrix();

  glDisable(GL_LIGHTING);

  double* look_vec=GenericViewer::camera.lookat.get_vec();
  glColor3f(0.7,0.7,1);  
  glBegin(GL_LINES);
  glVertex3f(0,0,0);
  glVertex3f(look_vec[0],look_vec[1],look_vec[2]); 
  glEnd();

  glTranslatef(look_vec[0],look_vec[1],look_vec[2]);


  glutWireSphere(0.07,4,4);
  glEnable(GL_LIGHTING);
  glPopMatrix();
 
  //glutSolidTeapot(0.5);
  return 0;
}

gint QuatFractViewer::gl_motion_notify_handler_imp(GdkEventMotion* event){
  switch (mouse_focus) {
  case 0:
    if( QuatFractViewer::finish_render)
      GenericViewer::gl_motion_notify_handler_imp(event);
    break;
  case 4:
    if( QuatFractViewer::finish_render);
    gfp->mouse_motion_imp(event);
    break;
  }
}


gint QuatFractViewer::gl_button_press_handler_imp(GdkEventButton* event){
  switch (mouse_focus) {
  case 0:
    if( QuatFractViewer::finish_render)
      GenericViewer::gl_button_press_handler_imp(event);
    break;
  case 4:
    if( QuatFractViewer::finish_render);
    gfp->mouse_press_imp(event);
    break;
  }
  
}

gint QuatFractViewer::gl_button_release_handler_imp(GdkEventButton* event){
  switch (mouse_focus) {
  case 0:
    if( QuatFractViewer::finish_render)
      GenericViewer::gl_button_release_handler_imp(event);
    break;
  case 4:
    if( QuatFractViewer::finish_render);
    gfp->mouse_release_imp(event);
    break;
  }
  
}


/* draw coordinate axes */
void QuatFractViewer::draw_coord() {

  glDisable(GL_LIGHTING);
  glBegin(GL_LINES);
    glColor3f(1, 0, 0);
    glVertex3f( 0, 0, 0);
    glVertex3f( 3, 0, 0);
    glVertex3f( 0, 0, 0);
    glVertex3f( 0, 3, 0);
    glVertex3f( 0, 0, 0);
    glVertex3f( 0, 0, 3);
  glEnd();
  output(3, 0, 0, "x");
  output(0, 3, 0, "y");
  output(0, 0, 3, "z");
  glEnable(GL_LIGHTING);
}


/* draw bounding box in which fractal is rendered */
void QuatFractViewer::draw_boundingbox() {

  FractParams fp = qf.get_fparams();
  
  glDisable(GL_LIGHTING);
  glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
  glBegin(GL_QUAD_STRIP);
  glColor3f(0, 1, 0);
    glVertex3f( fp.xMin, fp.yMin, fp.zMin);
    glVertex3f( fp.xMax, fp.yMin, fp.zMin);
    glVertex3f( fp.xMin, fp.yMax, fp.zMin);
    glVertex3f( fp.xMax, fp.yMax, fp.zMin);
    glVertex3f( fp.xMin, fp.yMax, fp.zMax);
    glVertex3f( fp.xMax, fp.yMax, fp.zMax);
    glVertex3f( fp.xMin, fp.yMin, fp.zMax);
    glVertex3f( fp.xMax, fp.yMin, fp.zMax);
    glVertex3f( fp.xMin, fp.yMin, fp.zMin);
    glVertex3f( fp.xMax, fp.yMin, fp.zMin);

  glEnd();
  glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
  glEnable(GL_LIGHTING);

}



void* enter_thread(void *pthis) {

  QuatFractCreator* qf = (QuatFractCreator*) pthis;

  qf->init_conv_map();
  qf->cube_polygonize();

  QuatFractViewer::finish_render = false;
  glNewList(1, GL_COMPILE);
  qf->cube_merge_n_draw();
  glEndList();
  QuatFractViewer::finish_render = true;

  pthread_exit(0);
}

