/*
 *  QuatMath.cc
 *
 *  Quaternion mathematics
 *
 *  Copyright (C) 2002 Tobias Dyckerhoff
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#include "QuatMath.h"

Quat::Quat() {}


Quat::Quat(double *ar) {
  for( int i = 0; i < 4; i++)
    vec[i] = ar[i];
}

Quat::Quat(double a, double b, double c, double d) {
  vec[0] = a;
  vec[1] = b;
  vec[2] = c;
  vec[3] = d;
}

bool Quat::operator==(const Quat &q) const {
  for( int i = 0; i < 4; i++)
    if( vec[i] != q.vec[i])
      return false;
  return true;
}


bool Quat::operator!=(const Quat &q) const {
  return !(*this == q);
}

Quat& Quat::operator=(const Quat &q) {
  for(int i = 0; i < 4; i++)
    vec[i] = q.vec[i];
}


Quat& Quat::operator*(const Quat &q) const {
  Quat *res = new Quat();
  res->vec[0] = vec[0]*q.vec[0] 
    - vec[1]*q.vec[1] 
    - vec[2]*q.vec[2]
    - vec[3]*q.vec[3];

  res->vec[1] = vec[0]*q.vec[1]
    + vec[1]*q.vec[0]
    + vec[2]*q.vec[3]
    - vec[3]*q.vec[2];

  res->vec[2] = vec[0]*q.vec[2]
    + vec[2]*q.vec[0]
    + vec[3]*q.vec[1]
    - vec[1]*q.vec[3];

  res->vec[3] = vec[0]*q.vec[3]
    + vec[3]*q.vec[0]
    + vec[1]*q.vec[2]
    - vec[2]*q.vec[1];
  
  return *res;
}

  

Quat& Quat::operator+(const Quat &q) const {
  Quat *res = new Quat();

  for( int i = 0; i < 4; i++) 
    res->vec[i] = vec[i] + q.vec[i];

  return *res;
}

Quat& Quat::operator-(const Quat &q) const {
  Quat *res = new Quat();

  for( int i = 0; i < 4; i++) 
    res->vec[i] = vec[i] - q.vec[i];

  return *res;
}


double Quat::abs() const {
  return sqrt(vec[0]*vec[0]
    + vec[1]*vec[1]
    + vec[2]*vec[2]
    + vec[3]*vec[3]);
}

Quat& Quat::inv() const {
  Quat *res = new Quat();
  double abs2 = vec[0]*vec[0]
    + vec[1]*vec[1]
    + vec[2]*vec[2]
    + vec[3]*vec[3];
  
  //  cout << abs2 << endl;

  res->vec[0] = vec[0]/abs2;
  for( int i = 1; i < 4; i++) 
    res->vec[i] = -vec[i]/abs2;
  
  return *res;
}


double* Quat::get_vec() const {
  double *ar = new double[4];
  for( int i = 0; i < 4; i++)
    ar[i] = vec[i];

  return ar;
}


ostream& operator<<(ostream &s, const Quat &q) {
  double *v = q.get_vec();
  
  s << v[0] << " + ";
  s << "i*" << v[1] << " + ";
  s << "j*" << v[2] << " + ";
  s << "k*" << v[3];
  
  return s;
}
