/*
 *  Vector3.h
 *
 *  Mathematical 3 dim vector class
 *
 *  Copyright (C) 2002 Tobias Dyckerhoff
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


//was solln da nicht inline sein?

#ifndef VECTOR3_H
#define VECTOR3_H

#include <iostream.h>
#include <math.h>


/** 3D Vector class.
    Simple 3D vector class with some operations.*/

class Vector3 {
  /** The array. */ 
  double vec[4]; 
  
 public:
  /** Constructor. */
  Vector3();

  /** Array constructor. Accepting the 3 doubles saved in an array.*/
  Vector3(double *array);

  /** Element constructor. Accepting the 3 doubles given separately.*/
  Vector3(double a, double b, double c);
  

  /** Equal. this == q ?*/
  bool operator==(const Vector3 &q) const;

  /** Unequal. this != q ?*/
  bool operator!=(const Vector3 &q) const;

  /** Assignment. this = q */
  Vector3& Vector3::operator=(const Vector3 &q);

  /** Multiplication. this * q */
  double operator*(const Vector3 &q) const; //scalar product

  /** Scalar Multiplication. Overloaded: Multiplication with a scalar: this * s */
  Vector3& operator*(const double &s) const; //scalar multiplication

  /**Division by a scalar. this / s 
   No checking for division by zero !*/
  Vector3& operator/(const double &s) const; 
  
  /** Vector Cross product. this x q */
  Vector3& operator^(const Vector3 &q) const; 
  
  /** Addition. this + q */
  Vector3& operator+(const Vector3 &q) const;
  
  /** Subtraction=. this -= q */
  Vector3& operator-(const Vector3 &q) const;
  
  /** Multiplication= with a scalar! this -= q */
  Vector3& operator*=(const double &s);

  /** Addition=. this += q */
  Vector3& operator+=(const Vector3 &q);

  /** Access to component.*/  
  double& operator[](int i);

  /** Norm. ||this||*/
  double abs() const;

  /** Get the coordinate vector. */
  double* get_vec() const;

};

inline Vector3& Vector3::operator+=(const Vector3 &q) {
  Vector3 *res = new Vector3();

  for( int i = 0; i < 3; i++) 
    res->vec[i] = vec[i] + q.vec[i];

  *this = *res;
  delete res;
  return *this;
}

inline Vector3& Vector3::operator*=(const double &s) {
  Vector3 *res = new Vector3();
  res->vec[0] = vec[0]*s; 
  
  res->vec[1] = vec[1]*s;
    
  res->vec[2] = vec[2]*s;


  
  *this = *res;

  delete res;
  return *this;
}


inline Vector3& Vector3::operator*(const double &s) const {
  Vector3 *res = new Vector3();
  res->vec[0] = vec[0]*s ;
  res->vec[1] = vec[1]*s;
  res->vec[2] = vec[2]*s;
return *res;
}

inline Vector3& Vector3::operator/(const double &s) const {
  Vector3 *res = new Vector3();
  res->vec[0] = vec[0]/s ;
  res->vec[1] = vec[1]/s;
  res->vec[2] = vec[2]/s;
return *res;
}



inline double Vector3::operator*(const Vector3 &q) const {
  return  vec[0]*q.vec[0]+vec[1]*q.vec[1]+vec[2]*q.vec[2];
}

inline Vector3& Vector3::operator^(const Vector3 &q) const {
 Vector3 *res = new Vector3();
 res->vec[0]=vec[1]*q.vec[2]-vec[2]*q.vec[1];
 res->vec[1]=vec[2]*q.vec[0]-vec[0]*q.vec[2];
 res->vec[2]=vec[0]*q.vec[1]-vec[1]*q.vec[0];
 return *res;
}



inline Vector3& Vector3::operator+(const Vector3 &q) const {
  Vector3 *res = new Vector3();

  for( int i = 0; i < 3; i++) 
    res->vec[i] = vec[i] + q.vec[i];

  return *res;
}

inline Vector3& Vector3::operator-(const Vector3 &q) const {
  Vector3 *res = new Vector3();

  for( int i = 0; i < 3; i++) 
    res->vec[i] = vec[i] - q.vec[i];

  return *res;
}



inline double Vector3::abs() const {
  return sqrt(vec[0]*vec[0]
	      + vec[1]*vec[1]
	      + vec[2]*vec[2]);
}


inline double& Vector3::operator[](int i){
  return vec[i];
}



ostream& operator<<(ostream &s, const Vector3 &q);

#endif VECTOR3_H
