/*
 *  Viewer.cc
 *
 *  The Viewer class used to create new specific viewers
 *
 *  Copyright (C) 2002 Tobias Dyckerhoff
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


 
//im augenblick ist viewer friend in gl_display, damit viewer mit dem realize_handler auf das realizekommando zugreifen kann 
 
//dynamische speicherverwaltung
//alles auf strings umschreiben!!
//text eingabe verfeinern (ein minus, ein punkt)...

#include "Viewer.h"


ViewerDisplay::ViewerDisplay() {
  /* this are calls to ensure correct mouse handling*/
  set_events(GDK_EXPOSURE_MASK|
	     GDK_BUTTON_MOTION_MASK|
	     GDK_POINTER_MOTION_MASK|
	     GDK_POINTER_MOTION_HINT_MASK|
	     GDK_BUTTON_PRESS_MASK|
	     GDK_BUTTON_RELEASE_MASK);
  
/* set the minimum size of the GLarea widget */
  set_usize(600,600);
}



Viewer::Viewer(const char* lab) :
  hbox(false, 0),vbox(false,0),upper_hbox(false,0),lower_hbox(false,0)
{
  /* initializing some member variables */

  gl_mouse1_pressed=0;
  gl_mouse2_pressed=0;
  gl_mouse3_pressed=0;
  
  gl_rot_x=0;
  gl_rot_y=0;
  gl_rot_z=0;
  gl_zoom=0;


  /* connect all display events, including the mouse events, to the appropiate 
     Viewer methods */ 

  display.expose_event.connect(slot(this,&Viewer::gl_expose_handler));
  display.realize.connect(slot(this,&Viewer::gl_realize_handler));
  display.configure_event.connect(slot(this,&Viewer::gl_configure_handler));
  
  display.motion_notify_event.connect(slot(this,&Viewer::gl_motion_notify_handler_imp));
  display.button_press_event.connect(slot(this,&Viewer::gl_button_press_handler_imp));
  display.button_release_event.connect(slot(this,&Viewer::gl_button_release_handler_imp));

  /* minimum window size */
  set_usize(800,700);
  
  set_title(lab);



  /* packing it all together */

  add(vbox);

  vbox.pack_start(upper_hbox);

  glframe.set_shadow_type (GTK_SHADOW_IN);
  glframe.set_border_width(6);  
  glframe.add(display);  
  vbox.pack_start(glframe);

  vbox.pack_start(lower_hbox);

  show_all();


  /* initializing and drawing the scene; any gl calls must be made AFTER the 
     widget is realized (which is the case because show_all() was called) */  

  updategl();
  initgl();
  drawgl();
}




void Viewer::initgl() {

  /* make_current must be called before performing GL operations */
   if (display.make_current()) {

     /* setting up the camera with some starting stats */

     Vector3 camera_start(1,1,4); 
     Vector3 lookat_start(0,0,0);
     Vector3 up_start(0,1,0);
     Vector3 light_start(1,3,3);

     camera.set_pos(camera_start);
     camera.set_lookat(lookat_start);
     camera.set_up(up_start);


     /*doing all the light stuff */

     set_light(light_start);
     
     GLfloat light_vec[4];
     for (int i=0;i<=2;i++)  light_vec[i]=gl_light_pos[i];
     light_vec[3]=0;
     
     GLfloat light_ambient[] = { 0.1, 0.1, 0.1, 1.0 };
     GLfloat light_diffuse[] = { 1, 1, 1, 1.0 };
     GLfloat light_specular[] = { 0, 0, 0, 1.0 };

     glLightfv(GL_LIGHT0, GL_AMBIENT, light_ambient);
     glLightfv(GL_LIGHT0, GL_DIFFUSE, light_diffuse);
     glLightfv(GL_LIGHT0, GL_SPECULAR, light_specular);
     glLightfv(GL_LIGHT0, GL_POSITION, light_vec);
     
     GLfloat mat_specular[] = { 0., 0., 0., 1.0 };
     GLfloat mat_diffuse[] = { 0.3, 0.3, 0.3, 1.0 };
     
     glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, mat_specular);
     glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, mat_diffuse);
     
     glColorMaterial(GL_FRONT_AND_BACK, GL_DIFFUSE);
     glEnable(GL_COLOR_MATERIAL);  
     
     glEnable(GL_LIGHTING);
     glEnable(GL_LIGHT0);
     

     /* other GL stuff */

     glEnable(GL_DEPTH_TEST);
     glShadeModel(GL_SMOOTH);  
     
 
     /* were done so we call updategl() */
     
     updategl();
     
     
     /* calling the virtual init_gl_context_imp */

     init_gl_context_imp();
     
   }
}

 


void Viewer::updategl(){
  if ( display.make_current()){

    /* updating GL environment */
   
    glClearColor(0.0, 0.0, 0.0, 0.0); 
   
    glViewport(0, 0, (GLint)(width()), (GLint)(height()));

    /*
      if (width()<=height()) glViewport(0, 0, (GLint)(width()), (GLint)(width()));
      else glViewport(0, 0, (GLint)(height()), (GLint)(height()));
    */

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(30.0, ((double)width()/height()), 1.0,200.0);

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();


    /* now the rotation	of the scene will be done by changing the camera stats.
       gl_rot_y etc. are set by the mouse event handler methods */

    if(gl_rot_y!=0)
      {

	/* rotate around an axis parallel to the up-axis through origin */
	
	Vector3 camera_dir=camera.pos-camera.lookat;
	camera_dir=q_rotate(camera_dir, gl_rot_y/200, camera.up);
	camera.pos=camera.lookat+camera_dir;
	
      }
    
    if (gl_rot_x!=0){
      
      /* rotate around the axis perpendicular to camera-lookat and up */
      
      Vector3 camera_dir=camera.pos-camera.lookat;
      Vector3 axis=camera_dir^camera.up/camera_dir.abs();
      camera_dir=q_rotate(camera_dir, gl_rot_x/200, axis);
      camera.up=q_rotate(camera.up, gl_rot_x/200, axis);
      camera.pos=camera.lookat+camera_dir;
      
    }
    
    
    if (gl_rot_z!=0){
      
      /* roll around */
      
      Vector3 camera_dir=camera.pos-camera.lookat;
      Vector3 axis=camera_dir/camera_dir.abs();
      camera.up=q_rotate(camera.up, gl_rot_z/600, axis);
    }
    
    
    /* zooming towards the lookat */

    if (gl_zoom!=0)
      {
	
	Vector3 dir=camera.pos-camera.lookat;
     
	Vector3 change=dir*(gl_zoom/500);
	camera.set_pos(camera.pos+change);
      }
    
    /* Now the new stats are applied to the gluLookAt */
    
    GLfloat light_vec[4];
    double* camera_vec=camera.pos.get_vec();
    double* lookat_vec=camera.lookat.get_vec();
    double* up_vec=camera.up.get_vec();
    
    for (int i=0;i<=3;i++)  light_vec[i]=gl_light_pos.get_vec()[i];
    
    gluLookAt(camera_vec[0],camera_vec[1],camera_vec[2],
	      lookat_vec[0],lookat_vec[1],lookat_vec[2],
	      up_vec[0],up_vec[1],up_vec[2]);
    
    glLightfv(GL_LIGHT0, GL_POSITION, light_vec);


    /* If some camera property has changed the gl_changed_event is emmitted */

    if (gl_rot_x!=0||gl_rot_y!=0||gl_rot_z!=0||gl_zoom!=0)
      {
	gl_changed_event();
      }
    
  }
  
}





gint Viewer::drawgl() {

  /* when starting opengl rendering,all opengl calls must be inside 
     make_current */

  if (display.make_current()) {
   
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

    glPushMatrix();

    /* calling the virtual context method*/
    draw_gl_context_imp(); 

    glPopMatrix();
  }
  
  display.swap_buffers();
}



gint Viewer::init_gl_context_imp(){
  /* nothing to be done here by default */
  return 0;
}

gint Viewer::draw_gl_context_imp(){
  /* nothing to be done here by default */
  return 0;
}



gint  Viewer::gl_expose_handler(GdkEventExpose* expose){
  /* we are CPU intensive so we should wait for the last event */
  /* well this comment is from somewhere else, actually i dont know what happends
     here exactly */
   if (expose->count>0) return 0;
   drawgl();
   return TRUE;
}

gint Viewer::gl_configure_handler(GdkEventConfigure* configure){
   if (display.make_current())
     {
      updategl();
     }
 return TRUE;
}


void Viewer::gl_realize_handler(){
  /* the opengl widget must be realized before any gl calls are made. Thus, its
     realize method is called here. To make this possible the Viewer was declared
     Friend Class to the ViewerDisplay */
     
  display.realize_impl();
  initgl(); 
}



gint Viewer::gl_motion_notify_handler_imp(GdkEventMotion* event){
  gint x,y;
  
  GdkModifierType state;

  /* the hint-thing is about filtering out some unnecessary events */
  
  if (event->is_hint)
    gdk_window_get_pointer (event->window, &x, &y, &state);
  else
    {
      x = (gint) event->x;
      y = (gint) event->y;
      state =(GdkModifierType) event->state;
    }
  
  /* now all the gl_rot.. variables are set. */

  if ((state & GDK_BUTTON1_MASK)&&gl_mouse1_pressed)
   {
     gl_rot_x=y-gl_mouse_start_y;
     gl_mouse_start_y=y;
     gl_rot_y=-(x-gl_mouse_start_x);
     gl_mouse_start_x=x;
     
     updategl();
     drawgl();
   }
 if ((state & GDK_BUTTON2_MASK)&&gl_mouse2_pressed)
   {
     gl_rot_z=x-gl_mouse_start_x;
     gl_mouse_start_x=x;

     updategl();
     drawgl();
   }

 if ((state & GDK_BUTTON3_MASK)&&gl_mouse3_pressed)
   {
     gl_zoom=+y-gl_mouse_start_y;
     gl_mouse_start_y=y;
     
     updategl();
     drawgl();
   }
 
 return TRUE;
}



gint Viewer::gl_button_press_handler_imp(GdkEventButton* event){
  if (event->button==1){
  gl_mouse_start_x=event->x;
  gl_mouse_start_y=event->y;
  gl_mouse1_pressed=1;
  }
 if (event->button==2){
  gl_mouse_start_x=event->x;
  gl_mouse_start_y=event->y;
  gl_mouse2_pressed=1;
  }

 if (event->button==3){
  gl_mouse_start_x=event->x;
  gl_mouse_start_y=event->y;
  gl_mouse3_pressed=1;
  }
}



gint Viewer::gl_button_release_handler_imp(GdkEventButton* event){
  if (event->button==1)  
    {
     gl_mouse1_pressed=0;
     gl_rot_x=0;
     gl_rot_y=0;
    }

 if (event->button==2)  
   {
     gl_mouse2_pressed=0;
     gl_rot_z=0;
   }

 if (event->button==3)  
   {
     gl_mouse3_pressed=0;
     gl_zoom=0;
   }

}


int Viewer::set_light(Vector3 pos){
  gl_light_pos=pos;
}





