/*
 *  permutations.h
 *
 *  Permutation mathematics
 *
 *  Copyright (C) 2002 Tobias Dyckerhoff
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#ifndef PERMUTATIONS_H
#define PERMUTATIONS_H

#include <iostream.h>
#include <stdio.h>

typedef unsigned int perm_t;
typedef unsigned int size_t;


/** Permutation class.
    Mathematical interface to a permutation providing the standard
    operation and some extra stuff.
*/
class Perm {
  perm_t *map_array;
  size_t size;

public:

  /** Constructor. Create permutation in S_ssize with mappings mmap. */
  Perm(size_t ssize, perm_t *mmap);
  /** Constructor. Create identity permutation in S_ssize. */
  Perm(size_t size);
  /** Constructor. Create identity in S_5. */
  Perm();
  
  /** Operation. Get result of the operation on el. */
  perm_t map(perm_t el) const;

  /** Size. Get the n in S_n. */
  size_t get_size() const;

  /** Equal. this == p ?*/
  bool operator==(const Perm &p) const;

  /** Unequal. this != p ?*/
  bool operator!=(const Perm &p) const;

  /** Multiplication. this * p*/
  Perm& operator*(const Perm &p) const;

  /** Print. Nice output of this.*/
  void print() const;

  /** Sign. Return sign of this.*/
  int sign() const;

  /** Order. Return order of this in S_n.*/
  int ord() const;

  /** Inversion. Return this^(-1).*/
  Perm& inv() const;

};

/** Standard c++ output for permutations. */
ostream& operator<<(ostream &s, const Perm &p);

#endif    



  
