/*
 *  Gtk_CameraControl.cc
 *
 *  Camera Control widget
 *
 *  Copyright (C) 2002 Tobias Dyckerhoff
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#include "Gtk_CameraControl.h"

using SigC::bind;
using SigC::slot;



CameraControl::CameraControl():
  vbox(false,0), camera_Gtk_CoordEntry("camera position",1),lookat_Gtk_CoordEntry("lookat position",2), up_Gtk_CoordEntry("up direction",3), light_Gtk_CoordEntry("light position",4), move_mode_button1("move"), rotate_mode_button2("rotate\n&zoom")
{


  /* Connecting the buttons. */

  move_mode_button1.toggled.connect( bind<int>(slot(this, &CameraControl::on_button_toggled), 1));
  rotate_mode_button2.toggled.connect( bind<int>(slot(this, &CameraControl::on_button_toggled), 2));


  /* packing it all together */

  hbox.pack_start(move_mode_button1);
  hbox.pack_start(rotate_mode_button2);

  button_frame.set_shadow_type (GTK_SHADOW_ETCHED_IN);
  button_frame.set_border_width(6);  
  button_frame.set_label("mouse movement");
  button_frame.add(hbox);  

  vbox.pack_start(button_frame);
  vbox.pack_start(camera_Gtk_CoordEntry);
  vbox.pack_start(lookat_Gtk_CoordEntry);
  vbox.pack_start(up_Gtk_CoordEntry);
  vbox.pack_start(light_Gtk_CoordEntry);
  add(vbox);



  /** in the beginning button2 (and mode 2) are active. make sure that the mode
      is set accordingly in the GenericViewer! */
  rotate_mode_button2.set_active(true); 

  /** window is hiding in the beginning */
  active=0;

  set_title("Camera");
}


void CameraControl::on_button_toggled(int id)
{
  
  /* only one of the buttons is active at the same time. mode_set signal is 
     emitted when the other one is pressed */

  if (id==1&&move_mode_button1.get_active()) 
    {
      rotate_mode_button2.set_active(false);
      mode_set(1);
    }


  if (id==2&&rotate_mode_button2.get_active())
    {
      move_mode_button1.set_active(false);
      mode_set(2);
    }

  if (!move_mode_button1.get_active()&&!rotate_mode_button2.get_active())
    {
      if (id==1)  move_mode_button1.set_active(true);
      else  rotate_mode_button2.set_active(true);
    }

}



/*i dont want the widget to be destroyed when closed but to go inactive and hide*/
int CameraControl::delete_event_impl(GdkEventAny* event)
{
  active=0;
  hide_all();

  /* returnig true means it wont be destroyed */
  return true;
}
