/*
 *  Gtk_PathMaster.cc
 *
 *  Master of waypoint path
 *
 *  Copyright (C) 2002 Tobias Dyckerhoff
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#include "Gtk_PathMaster.h"


using SigC::bind;
using SigC::slot;


Waypoint::Waypoint()
{
  x4=0;
  time=0;
}


int Waypoint::set_time(double p)
{
  time=p;
  time_update(time);
}

int Waypoint::set_x4(double p)
{
  x4=p;
  x4_update(x4);
}



WaypointEditor::WaypointEditor():
  table(6,3,false),
  get_cam_b("get stats from Camera"), 
  insert_b("insert waypoint"),
  replace_b("apply to\n current waypoint"),
  erase_b("erase waypoint"),
  next_wp_b("next"),
  prev_wp_b("prev."),
  pos("Position",1), 
  lookat("LookAt",2), 
  up("Up",3),
  x4("x4  ",4),
  time("time",5)
{ 
  vbbox.pack_start(insert_b);
  vbbox.pack_start(replace_b);
  vbbox.pack_start(erase_b);
  frame.set_shadow_type (GTK_SHADOW_ETCHED_IN);
  frame.set_border_width(6);  
  frame.add(vbbox);  
  wp_entry.set_usize(100,-1);
  wp_entry.set_editable(false);
  vwpbox.pack_start(next_wp_b);
  vwpbox.pack_start(wp_entry);
  vwpbox.pack_start(prev_wp_b);
  wpframe.set_shadow_type (GTK_SHADOW_ETCHED_IN);
  wpframe.set_border_width(6);  
  wpframe.add(vwpbox);  

  table.attach(get_cam_b,0,1,0,1);
  table.attach(x4,0,1,1,2);
  table.attach(time,0,1,2,3);
  table.attach(pos,1,2,0,3);
  table.attach(lookat,2,3,0,3);
  table.attach(up,3,4,0,3);
  table.attach(frame,4,5,0,3);
  table.attach(wpframe,5,6,0,3);

  add(table);
  show_all();
}


/*
int WaypointEditor::delete_event_impl(GdkEventAny* event)
{

  toggle_my_tool();
  
  //returnig true means it wont be destroyed; it will be destroyed by toggeling the toolbarbutton, which itself will destroy it;
  return true;
}
*/


void WaypointEditor::update_coord_entries(const Waypoint& wp)
{
 pos.set_coord(wp.pos);
 lookat.set_coord(wp.lookat);
 up.set_coord(wp.up); 
 time.set_value(wp.time);
 x4.set_value(wp.x4);
 }

PathMaster::PathMaster()
{
  active=0;

  /* current=-1 means that the path empty */
  current=-1;
}
 
void PathMaster::add_waypoint()
{
  


  Waypoint* wp= new Waypoint;
  wp->pos=new_wp.pos;
  wp->lookat=new_wp.lookat;
  wp->up=new_wp.up;
  //SET THIS TO CURRENT X4 OF THE FRVIEWER
  wp->x4=new_wp.x4;

  current++;

  wp->time =new_wp.time ;   

  /* adding new wp */  
  path.insert(path.begin()+current,wp);

  /* setting the entries to current wps stats */
  char j[11];
  sprintf(j,"WayPoint%i",current);
  editor->wp_entry.set_text(j);
  editor->update_coord_entries(*path[current]); 

  /* emit neccesary signals */
  redraw(); 
  path_update();
}

void PathMaster::delete_waypoint()
{

  /* kind of ugly but seems to work */

  if (path.empty()) return ;
  current--;
  if(path.size()>1||current==0){
    path.erase(path.begin() +current+1);
    if (path.size()>0&&current==-1) current++;
  }
  else{
    path.erase(path.begin() +current+1);
  }
  char j[11];
  if (current==-1) sprintf(j,"",current);
  else sprintf(j,"WayPoint%i",current);

  editor->wp_entry.set_text(j);

  if (current!=-1)
    {
      new_wp=*path[current];
      editor->update_coord_entries(*path[current]); 
    }


  /* emit neccesary signals */
     
   redraw();
   path_update();
}


void PathMaster::replace_waypoint()
{
  if (path.empty()||current==-1) return;

  *path[current]=new_wp;
   redraw();
   path_update();
}

void PathMaster::activate()
{
  if(active){editor->destroy(); return ;}
  active=1;


  editor= manage (new WaypointEditor);

  /* the new WaypointEditor window needs to be initialized again */

  editor->destroy.connect(slot(this,&PathMaster::deactivate));
  editor->insert_b.clicked.connect(slot(this,&PathMaster::add_waypoint));
  editor->replace_b.clicked.connect(slot(this,&PathMaster::replace_waypoint));
  editor->erase_b.clicked.connect(slot(this,&PathMaster::delete_waypoint));
  editor->next_wp_b.clicked.connect(slot(this,&PathMaster::next_waypoint));  
  editor->prev_wp_b.clicked.connect(slot(this,&PathMaster::prev_waypoint));
  editor->get_cam_b.clicked.connect(slot(this,&PathMaster::get_camera_pressed));
  editor->pos.entry_activated.connect(slot(this,&PathMaster::entry_activated));
  editor->lookat.entry_activated.connect(slot(this,&PathMaster::entry_activated));
  editor->up.entry_activated.connect(slot(this,&PathMaster::entry_activated));
  editor->x4.entry_activated.connect(slot(this,&PathMaster::entry_activated));
  editor->time.entry_activated.connect(slot(this,&PathMaster::entry_activated));


  /* connecting changes of our new_wp to the text entries. */

  new_wp.pos_update.connect(slot(&editor->pos,&CoordEntry::set_coord));
  new_wp.lookat_update.connect(slot(&editor->lookat,&CoordEntry::set_coord));
  new_wp.up_update.connect(slot(&editor->up,&CoordEntry::set_coord));
  new_wp.x4_update.connect(slot(&editor->x4,&FloatEntry::set_value));
  new_wp.time_update.connect(slot(&editor->time,&FloatEntry::set_value));

  editor->set_title("Waypoint Editor");

  /* setting up the entries with the stats of the "current" waypoint if path is
     not empty */
  if(current!=-1){
    new_wp.set_pos(path[current]->pos); 
    new_wp.set_lookat(path[current]->lookat); 
    new_wp.set_up(path[current]->up);  
    new_wp.set_time(path[current]->time);
    new_wp.set_x4(path[current]->x4);

    editor->update_coord_entries(new_wp); 
    char j[11];
    if (current==-1) sprintf(j,"",current);
    else sprintf(j,"WayPoint%i",current);
    editor->wp_entry.set_text(j);
  }
  

  /* done */  
  editor->show_all();
}


void PathMaster::get_camera_pressed()
{
  SpaceStats test;
  test=camera_stats_input();
  new_wp.set_pos(test.pos); 
  new_wp.set_lookat(test.lookat); 
  new_wp.set_up(test.up); 
  if (current!=-1) 
    {
     new_wp.set_time(path[current]->time+1);
     new_wp.set_x4(path[current]->x4);
    }
  editor->update_coord_entries(new_wp); 
}


void PathMaster::entry_activated(int id)
{
  new_wp.set_up(editor->up.get_coord());

  new_wp.set_pos(editor->pos.get_coord());
   
  new_wp.set_lookat(editor->lookat.get_coord());

  new_wp.set_time(editor->time.get_value());

  new_wp.set_x4(editor->x4.get_value());
}


void PathMaster::deactivate()
{
  active=0;
}


void PathMaster::next_waypoint()
{
  if (current==path.size()-1) return;
    current++;
  char j[11];
   sprintf(j,"WayPoint%i",current);
   editor->wp_entry.set_text(j);
   editor->update_coord_entries(*path[current]); 

   /* emit redraw signal to display the new selection */
   redraw();
}

void PathMaster::prev_waypoint()
{
  if(current<=0) return;
  current--;
  char j[11];
   sprintf(j,"WayPoint%i",current);
   editor->wp_entry.set_text(j);
   editor->update_coord_entries(*path[current]); 

   /* emit redraw signal to display the new selection */
   redraw();
}


//own objekt for the gl stuff? display lists?
void PathMaster::draw()
{
  for(int i=0;i<path.size();i++)
    {
      glEnable( GL_NORMALIZE );
      glPushMatrix();
      glTranslatef(path[i]->pos[0],path[i]->pos[1],path[i]->pos[2]);
      glColor3f(1,0,0);
      Vector3 look0=(path[i]->lookat-path[i]->pos);
      Vector3 look=look0/look0.abs();
      Vector3 side=  path[i]->up ^ look;

      /* drawing the ugly waypoint */

      glBegin(GL_TRIANGLES);
      //got to simplify this
      Vector3 normal=(path[i]->up/10+(path[i]->up/10-side/8))^look;
      glNormal3f(normal[0],normal[1],normal[2]);
      glVertex3f(path[i]->up[0]/10,path[i]->up[1]/10,path[i]->up[2]/10);
      glNormal3f(normal[0],normal[1],normal[2]);
      glVertex3f((-path[i]->up[0]/10+side[0]-look[0])/8,
		 (-path[i]->up[1]/10+side[1]-look[1])/8,
		 (-path[i]->up[2]/10+side[2]-look[2])/8);
      glNormal3f(normal[0],normal[1],normal[2]);
      glVertex3f((-path[i]->up[0]/10+side[0]+look[0])/8,
		 (-path[i]->up[1]/10+side[1]+look[1])/8,
		 (-path[i]->up[2]/10+side[2]+look[2])/8);


      normal=side ^ (path[i]->up/10+(path[i]->up/10-look/8));
      glNormal3f(normal[0],normal[1],normal[2]);
      glVertex3f(path[i]->up[0]/10,path[i]->up[1]/10,path[i]->up[2]/10);
      glNormal3f(normal[0],normal[1],normal[2]);
      glVertex3f((-path[i]->up[0]/10+side[0]+look[0])/8,
		 (-path[i]->up[1]/10+side[1]+look[1])/8,
		 (-path[i]->up[2]/10+side[2]+look[2])/8);
      glNormal3f(normal[0],normal[1],normal[2]);
      glVertex3f((-path[i]->up[0]/10-side[0]+look[0])/8,
		 (-path[i]->up[1]/10-side[1]+look[1])/8,
		 (-path[i]->up[2]/10-side[2]+look[2])/8);

      normal=look^(path[i]->up/10+(path[i]->up/10+side/8));
      glNormal3f(normal[0],normal[1],normal[2]);
      glVertex3f(path[i]->up[0]/10,path[i]->up[1]/10,path[i]->up[2]/10);
      glNormal3f(normal[0],normal[1],normal[2]);
      glVertex3f((-path[i]->up[0]/10-side[0]+look[0])/8,
		 (-path[i]->up[1]/10-side[1]+look[1])/8,
		 (-path[i]->up[2]/10-side[2]+look[2])/8);
      glNormal3f(normal[0],normal[1],normal[2]);
      glVertex3f((-path[i]->up[0]/10-side[0]-look[0])/8,
		 (-path[i]->up[1]/10-side[1]-look[1])/8,
		 (-path[i]->up[2]/10-side[2]-look[2])/8);


      normal=(path[i]->up/10+(path[i]->up/10+look/8))^side;
      glNormal3f(normal[0],normal[1],normal[2]);
      glVertex3f(path[i]->up[0]/10,path[i]->up[1]/10,path[i]->up[2]/10);
      glNormal3f(normal[0],normal[1],normal[2]);
      glVertex3f((-path[i]->up[0]/10-side[0]-look[0])/8,
		 (-path[i]->up[1]/10-side[1]-look[1])/8,
		 (-path[i]->up[2]/10-side[2]-look[2])/8);
      glNormal3f(normal[0],normal[1],normal[2]);
      glVertex3f((-path[i]->up[0]/10+side[0]-look[0])/8,
		 (-path[i]->up[1]/10+side[1]-look[1])/8,
		 (-path[i]->up[2]/10+side[2]-look[2])/8);

      glEnd();

      glBegin(GL_QUADS);
      normal=path[i]->up/(-10);
      glNormal3f(normal[0],normal[1],normal[2]);
      glVertex3f((-path[i]->up[0]/10+side[0]-look[0])/8,
		 (-path[i]->up[1]/10+side[1]-look[1])/8,
		 (-path[i]->up[2]/10+side[2]-look[2])/8);
      glNormal3f(normal[0],normal[1],normal[2]);
      glVertex3f((-path[i]->up[0]/10-side[0]-look[0])/8,
		 (-path[i]->up[1]/10-side[1]-look[1])/8,
		 (-path[i]->up[2]/10-side[2]-look[2])/8);
      glNormal3f(normal[0],normal[1],normal[2]);
      glVertex3f((-path[i]->up[0]/10-side[0]+look[0])/8,
		 (-path[i]->up[1]/10-side[1]+look[1])/8,
		 (-path[i]->up[2]/10-side[2]+look[2])/8);
      glNormal3f(normal[0],normal[1],normal[2]);
      glVertex3f((-path[i]->up[0]/10+side[0]+look[0])/8,
		 (-path[i]->up[1]/10+side[1]+look[1])/8,
		 (-path[i]->up[2]/10+side[2]+look[2])/8);

      glEnd();

      glDisable(GL_LIGHTING);
      glBegin(GL_LINES);
      glVertex3f(0,0,0);
      glVertex3f(look0[0],look0[1],look0[2]);
      glEnd();	


      /* drawing the ugly selection box */
      if(i==current&&active==1)
	{
	  glColor3f(0,1,0);
	  glutWireCube(0.2);
      }
      glEnable(GL_LIGHTING);
      glPopMatrix();
      glDisable( GL_NORMALIZE );
    }
}





//don't look
void PathMaster::register_data (SigC::ScopeNode *)
{
 
}

