/*
 *  Gtk_PovrayOut.cc
 *
 *  Povray Output window
 *
 *  Copyright (C) 2002 Tobias Dyckerhoff
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#include "Gtk_PovrayOut.h"


#define DEBUG 1
#define FLT_SIZE 7 


using SigC::bind;
using SigC::slot;


CoordEntry1::CoordEntry1(const char* lab, int id):Table(2,3,false),coord(0,0,0),x_entry(5),y_entry(5),z_entry(5), x_label(" x "), y_label(" y "), z_label(" z "), label(lab)
{

  identifier=id;

  attach(x_label,0,1,0,1);
  attach(y_label,0,1,1,2);
  attach(z_label,0,1,2,3);


  attach(x_entry,1,2,0,1);
  attach(y_entry,1,2,1,2);
  attach(z_entry,1,2,2,3);


  x_entry.set_usize(80, -1);
  y_entry.set_usize(80, -1);
  z_entry.set_usize(80, -1);

  x_entry.activate.connect(slot(this, &CoordEntry1::on_entry_activated));
  y_entry.activate.connect(slot(this, &CoordEntry1::on_entry_activated));
  z_entry.activate.connect(slot(this, &CoordEntry1::on_entry_activated));

}



void CoordEntry1::on_entry_activated() {

  char st[10];
  int j;
  

  for (j=0;j<(x_entry.get_text()).length();j++) st[j]=(x_entry.get_text())[j]; 
  st[j]=0;

  for (int i=0;i<=strlen(st)-1;i++)
    {
      if ((st[i]<48||st[i]>57)&&st[i]!='.'&&st[i]!='-')
	{
	  set_coord(coord);
	  return;
	}
    }
  coord[0]=atof(st);

 for (j=0;j<(y_entry.get_text()).length();j++) st[j]=(y_entry.get_text())[j]; 
  st[j]=0;


  for (int i=0;i<=strlen(st)-1;i++)
    {

      if ((st[i]<48||st[i]>57)&&st[i]!='.'&&st[i]!='-')
	{
	  set_coord(coord);
	  return;
	}
    }
  coord[1]=atof(st);

  for (j=0;j<(z_entry.get_text()).length();j++) st[j]=(z_entry.get_text())[j]; 
  st[j]=0;
  //  for (int i=0;i<=4;i++) st[i]=(z_entry.get_text())[i]; //hier ist entrylength die obergrenze!

  for (int i=0;i<=strlen(st)-1;i++)
    {
      if ((st[i]<48||st[i]>57)&&st[i]!='.'&&st[i]!='-')
	{
	  set_coord(coord);
	  return;
	}
    }
  coord[2]=atof(st);
  set_coord(coord);
  entry_activated(identifier);

}

int CoordEntry1::get_id(){
  return identifier;
}


int CoordEntry1::set_coord(Vector3 c)
{
  coord=c;
 
  char st[18];
  sprintf(st,"%lf",c[0]);
  x_entry.set_text(st);
  sprintf(st,"%lf",c[1]);
  y_entry.set_text(st);
  sprintf(st,"%lf",c[2]);
  z_entry.set_text(st);
  return 1;
}



Vector3 CoordEntry1::get_coord()
{
  return coord;
}



/* fills the lookup entries into tplfile using the marking !*****! */
string* fill_tpl( map<string, string*>& lookup, 
		  string tplstr, 
		  char marking = '!'){
  string *tpl = new string(tplstr);
  string::size_type i, j;

  while( (i = tpl->find('!')) != string::npos) {
    j = string(tpl->begin()+i+1, tpl->end()).find( '!');
    if( j != string::npos) {
      if( string* st = lookup[string(*tpl, i+1, j)])
	tpl->replace(i, j+2, *st);
      else {
	cerr << "error: template " << string(*tpl, i+1, j) 
	     << " not found in lookup map" << endl;
	return new string("NOT FOUND");
      }
    }
  }

  return tpl;
}


/* returns the povray ini section that uses clocked phases to 
   drive along our waypoint linestrip and set the parameters 
   according to clock and waypoints */
string* get_povray_path( vector<Waypoint*> path) {
  

  string tpl = "#range (!R1!, !R2!)\n";
  tpl += "\t#declare clock!INDEX! = clock - !R1!;\n";
  tpl += "\tlocation (1-clock!INDEX!/(!R2! - !R1!))*!SRC_VEC! + clock!INDEX!/(!R2! - !R1!)*!DST_VEC!\n";
  tpl +=  "#break\n";

  char c[FLT_SIZE];
  map<string, string*> path_map;
  string* pp = new string;
  string tmp;
  

  *pp = "#switch(clock)\n";

  for( int i = 0; i < path.size()-1; i++) {
    tmp = "";
    snprintf( c, FLT_SIZE, "%d", i);    
    tmp += c;
    path_map["INDEX"] =  new string(tmp);
    if(DEBUG) cout << "INDEX=" << tmp << endl;

    tmp = "";
    snprintf( c, FLT_SIZE, "%f", path[i]->time);    
    tmp += c;
    path_map["R1"] = new string(tmp);
    if(DEBUG) cout << "R1=" << tmp << endl;

    tmp = "";
    snprintf( c, FLT_SIZE, "%f", path[i+1]->time);    
    tmp += c;
    path_map["R2"] = new string(tmp);
    if(DEBUG) cout << "R2=" << tmp << endl;
    
    tmp = "<";
    snprintf( c, FLT_SIZE, "%f", path[i]->pos[0]);
    tmp += c;
    tmp += ",";
    snprintf( c, FLT_SIZE, "%f", path[i]->pos[1]);
    tmp += c;
    tmp += ",";
    snprintf( c, FLT_SIZE, "%f", path[i]->pos[2]);
    tmp += c;
    tmp += ">";
    path_map["SRC_VEC"] = new string(tmp);
    if(DEBUG) cout << "SRC_VEC=" << tmp << endl;

    tmp = "<";
    snprintf( c, FLT_SIZE, "%f", path[i+1]->pos[0]);
    tmp += c;
    tmp += ",";
    snprintf( c, FLT_SIZE, "%f", path[i+1]->pos[1]);
    tmp += c;
    tmp += ",";
    snprintf( c, FLT_SIZE, "%f", path[i+1]->pos[2]);
    tmp += c;
    tmp += ">";
    path_map["DST_VEC"] = new string(tmp);
    if(DEBUG) cout << "DST_VEC=" << tmp << endl;

    *pp += *fill_tpl(path_map, tpl);
  }

  *pp += "#end\n";

  return pp;
}


SceneSetupPage::SceneSetupPage(map<string, string*>& lookup, 
			       char* tplfile,
			       vector<Waypoint*> path) : 
  vbox1(false,0),
  hbox1(false,0),
  camera_Gtk_CoordEntry("",1),
  lookat_Gtk_CoordEntry("",2), 
  up_Gtk_CoordEntry("",3), 
  light_Gtk_CoordEntry("",4), 
  frame1("camera"),
  frame2("lookat"),
  frame3("up"),
  frame4("light"),
  button1("fill it"),
  button2("write file"),
  checkbox1("edit .pov file")
{


  this->lookup = lookup;
  this->tplfile = tplfile;
  filled = new string("-- template file not yet processed --\n");
  
  frame1.set_border_width(10);
  frame2.set_border_width(10);
  frame3.set_border_width(10);
  frame4.set_border_width(10);


  pack1(vbox1, FALSE, FALSE);
  pack2(sw, TRUE, FALSE);

  vbox1.pack_start(hbox1, true, false, 0);
  vbox1.pack_start(hbox2, false, false, 0);

  hbox1.pack_start(frame1);
  frame1.add(camera_Gtk_CoordEntry);

  hbox1.pack_start(frame2);
  frame2.add(lookat_Gtk_CoordEntry);

  hbox1.pack_start(frame3);
  frame3.add(up_Gtk_CoordEntry);

  hbox1.pack_start(frame4);
  frame4.add(light_Gtk_CoordEntry);


  hbox2.pack_start(button1, true, false, 0);
  hbox2.pack_start(button2, true, false, 0);
  hbox2.pack_start(checkbox1, true, false, 0);


  sw.set_border_width(5);
  sw.set_policy( GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
  sw.add( txt);

  txt.set_editable(false);
  txt.insert( *filled);



  camera_Gtk_CoordEntry.entry_activated.connect(slot(this,&SceneSetupPage::entry_activated));
  lookat_Gtk_CoordEntry.entry_activated.connect(slot(this,&SceneSetupPage::entry_activated));
  up_Gtk_CoordEntry.entry_activated.connect(slot(this,&SceneSetupPage::entry_activated));
  light_Gtk_CoordEntry.entry_activated.connect(slot(this,&SceneSetupPage::entry_activated));
 
  button1.clicked.connect( bind<int>(slot(this, &SceneSetupPage::on_button_clicked), 1));
  button2.clicked.connect( bind<int>(slot(this, &SceneSetupPage::on_button_clicked), 2));
  checkbox1.clicked.connect( slot(this, &SceneSetupPage::on_toggle));


}  



void SceneSetupPage::entry_activated(int id) {

  ostringstream buf;
  buf << camera_Gtk_CoordEntry.get_coord();
  string *buf1 = new string(buf.str());

  lookup["LOCATION"] = buf1;

}	  

void SceneSetupPage::on_button_clicked(int id) {

  ifstream tplstream( tplfile);
  string *tpl = new string;
  char* pov_out = "data/q.pov";

  switch(id) {
  case 1:

    if( !tplstream) {
      cerr << "error: cannot open " << tplfile << endl;
      *tpl = "";
    } else {
      getline(tplstream, *tpl, (char) -1);
    }

    txt.backward_delete(filled->size());


    filled = fill_tpl( lookup, *tpl);
    txt.insert(*filled);
    txt.set_editable(true);
    break;
  case 2:
    
    ofstream to(pov_out);
    if( !to) {
      cerr << "error: could not open " << pov_out << endl;
      break;
    }
    
    to << txt.get_chars(0, -1);
    to.close();

  }

}	  

void SceneSetupPage::on_toggle() {
  if( checkbox1.get_active()) {
    sw.set_usize( 400, 400);
    sw.show();
  } else {
    sw.set_usize( 0, 0);
    sw.hide();
  }
}	  

PovrayOptionsPage::PovrayOptionsPage(map<string, string*>& lookup, char* tplfile) : 
  vbox1(false,0),
  hbox1(false,0),
  button1("fill it"),
  button2("write file"),
  checkbox1("edit .ini file")
{

  list<char* > ll;
  GroupEntry* ge;

  ll.push_back( "dimensions");
  ll.push_back( "width");
  ll.push_back( "heigth");

  ge = manage( new GroupEntry(ll));
  ge->entry_activated.connect(bind<int >(slot( this, &PovrayOptionsPage::entry_activated), 0));

  /* some defaults */
  ge->set_entry(0, "320");
  ge->set_entry(1, "200");
  gentry_vec.push_back(ge);
  hbox1.pack_start(*ge);

  ll.clear();
  ll.push_back( "file options");
  ll.push_back( "filename");
  ll.push_back( "format");
  ll.push_back( "initial frame");
  ll.push_back( "final frame");

  ge = manage( new GroupEntry(ll, 80, 22, 20));
  ge->entry_activated.connect(bind<int >(slot( this, &PovrayOptionsPage::entry_activated), 1));
  /* some defaults */
  ge->set_entry(0, "q.pov");
  ge->set_entry(1, "TODO .tga");
  ge->set_entry(2, "1");
  ge->set_entry(3, "20");
  gentry_vec.push_back(ge);
  hbox1.pack_start(*ge);

  this->lookup = lookup;
  this->tplfile = tplfile;
  filled = new string("-- template file not yet processed --\n");


  pack1(vbox1, FALSE, FALSE);
  pack2(sw, TRUE, FALSE);

  vbox1.pack_start(hbox1, true, false, 0);
  vbox1.pack_start(hbox2, false, false, 0);


  hbox2.pack_start(button1, true, false, 0);
  hbox2.pack_start(button2, true, false, 0);
  hbox2.pack_start(checkbox1, true, false, 0);


  sw.set_border_width(5);
  sw.set_policy( GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
  sw.add( txt);

  txt.set_editable(false);
  txt.insert( *filled);

 
  button1.clicked.connect( bind<int>(slot(this, &PovrayOptionsPage::on_button_clicked), 1));
  button2.clicked.connect( bind<int>(slot(this, &PovrayOptionsPage::on_button_clicked), 2));
  checkbox1.clicked.connect( slot(this, &PovrayOptionsPage::on_toggle));

}  


void PovrayOptionsPage::entry_activated(int entry_id, int frame_id) {
  cout << gentry_vec[frame_id]->get_entry(entry_id) << endl;
}	  

void PovrayOptionsPage::on_button_clicked(int id) {

  ifstream tplstream( tplfile);
  string *tpl = new string;
  char* ini_out = "data/q.ini";

  switch(id) {
  case 1:
    if( !tplstream) {
      cerr << "error: cannot open " << tplfile << endl;
      *tpl = "";
    } else {
      getline(tplstream, *tpl, (char) -1);
    }

    lookup["WIDTH"] = new string(gentry_vec[0]->get_entry(0));
    lookup["HEIGHT"] = new string(gentry_vec[0]->get_entry(1));

    lookup["POV_FILE"] = new string(gentry_vec[1]->get_entry(0));
    lookup["INIT_FRAME"] = new string(gentry_vec[1]->get_entry(2));
    lookup["FINAL_FRAME"] = new string(gentry_vec[1]->get_entry(3));

    txt.backward_delete(filled->size());
    filled = fill_tpl( lookup, *tpl);
    txt.insert(*filled);
    txt.set_editable(true);
    break;
  case 2:

    ofstream to(ini_out);
    if( !to) {
      cerr << "error: could not open " << ini_out << endl;
      break;
    }
    
    to << txt.get_chars(0, -1);
    to.close();

  }
}	  



void PovrayOptionsPage::on_toggle() {
  if( checkbox1.get_active()) {
    sw.set_usize( 400, 400);
    sw.show();
  } else {
    sw.set_usize( 0, 0);
    sw.hide();
  }
}	  


void PovrayOptionsPage::update_endtime(double time) {
  char c[FLT_SIZE];
  string tmp;

  tmp = "";
  snprintf( c, FLT_SIZE, "%f", time);    
  tmp += c;

  lookup["ENDTIME"] = new string(tmp);
}

 
PovrayOut::PovrayOut( map<string, string*>& lookup, 
		      char* pov_tplfile,
		      char* ini_tplfile,
		      vector<Waypoint*> path) : 
  vbox3(false,0),
  ss_page( lookup, pov_tplfile, path),
  po_page( lookup, ini_tplfile)
{
  

  set_title("QuatFract Povray Output Generator");

  add(nb);

  nb.pages().push_back( Gtk::Notebook_Helpers::TabElem( ss_page, "Scene setup"));
  nb.pages().push_back( Gtk::Notebook_Helpers::TabElem( po_page, "Povray options"));
  nb.pages().push_back( Gtk::Notebook_Helpers::TabElem( vbox3, "Let's trace tha rays"));


  show_all();

  ss_page.on_toggle();
  po_page.on_toggle();

}



//we dont want the widget to be destroyed when closed but to go inactive and hide
int PovrayOut::delete_event_impl(GdkEventAny* event)
{
  //  active=0;
  hide_all();

  //returnig true means it wont be destroyed
  return true;
}
