/*
 *  Gtk_PovrayOut.h
 *
 *  Povray Output window
 *
 *  Copyright (C) 2002 Tobias Dyckerhoff
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#ifndef GTK_POVOUT_H 
#define GTK_POVOUT_H 


#include <fstream>
#include <sstream>
#include <string>
#include <map>
#include <list>
#include <vector>
#include <iostream>
#include <stdio.h>

#include <gtk--/box.h>
#include <gtk--/button.h>
#include <gtk--/entry.h>
#include <gtk--/main.h>
#include <gtk--/window.h>
#include <gtk--/table.h>
#include <gtk--/label.h>
#include <gtk--/separator.h>
#include <gtk--/text.h>
#include <gtk--/frame.h>
#include <gtk--/scrolledwindow.h>
#include <gtk--/notebook.h>
#include <gtk--/togglebutton.h>
#include <gtk--/checkbutton.h>
#include <gtk--/paned.h>
#include <gtk--/buttonbox.h>


#include "../Vector3.h"
#include "Gtk_GroupEntry.h"
#include "Gtk_PathMaster.h"

string* fill_tpl( map<string, string*>& lookup, 
		  char* tplfile,
		  char marking = '!');

string* get_povray_path( vector<Waypoint*> path);

class CoordEntry1: public Gtk::Table
{
  Gtk::Label label;

  Gtk::Entry x_entry;
  Gtk::Entry y_entry;
  Gtk::Entry z_entry;
  
  Gtk::Label x_label;
  Gtk::Label y_label;
  Gtk::Label z_label;


  void on_entry_activated();
  Vector3 coord;
  int identifier;

public:
  int get_id();
  SigC::Signal1<void, int> entry_activated;
  CoordEntry1(const char* lab, int id);
  int set_coord(Vector3 c);
  Vector3 get_coord();
};





class SceneSetupPage : public Gtk::VPaned {

  Gtk::VBox vbox1;
  Gtk::HBox hbox1;
  Gtk::HButtonBox hbox2;
  Gtk::HButtonBox hbox3;

  CoordEntry1 camera_Gtk_CoordEntry;
  CoordEntry1 lookat_Gtk_CoordEntry;
  CoordEntry1 up_Gtk_CoordEntry;
  CoordEntry1 light_Gtk_CoordEntry;
  
  Gtk::Frame frame1;
  Gtk::Frame frame2;
  Gtk::Frame frame3;
  Gtk::Frame frame4;

  Gtk::Button button1;
  Gtk::Button button2;

  Gtk::CheckButton checkbox1;

  Gtk::ScrolledWindow sw;
  Gtk::Text txt;
 
  gint delete_event_impl(GdkEventAny*) {
    Gtk::Main::quit(); 
    return 0;
  }

  map<string, string*> lookup;
  char* tplfile;
  string *filled;

  void entry_activated(int id);
  void on_button_clicked(int id);

public:

  void on_toggle();
  SceneSetupPage( map<string, string*>& lookup, 
		  char* tplfile,
		  vector<Waypoint*> path);
  void update_lookup(const string& k, string* str) {lookup[k] = str;}

};




class PovrayOptionsPage : public Gtk::VPaned {

  Gtk::VBox vbox1;
  Gtk::HBox hbox1;
  Gtk::HButtonBox hbox2;
  
  vector<GroupEntry* > gentry_vec;

  Gtk::Button button1;
  Gtk::Button button2;

  Gtk::CheckButton checkbox1;

  Gtk::ScrolledWindow sw;
  Gtk::Text txt;
 
  gint delete_event_impl(GdkEventAny*) {
    Gtk::Main::quit(); 
    return 0;
  }

  map<string, string*> lookup;
  char* tplfile;
  string *filled;


  void entry_activated(int entry_id, int frame_id);
  void on_button_clicked(int id);


public:

  void update_endtime(double time);
  void on_toggle();
  PovrayOptionsPage( map<string, string*>& lookup, char* tplfile);

};



/** Povray Ouput window.
    Within this window one can fill the waypoint path defined by the
    PathMaster into some povray template files and edit them if required.
    The filled templates will be able to render a film sequence with povray. */

class PovrayOut : public Gtk::Window {

  /* Gtk stuff */

  Gtk::Notebook nb;

  SceneSetupPage ss_page;
  PovrayOptionsPage po_page;
  Gtk::VBox vbox3;


  /* other data */

public:
  
  /** Constructor. accepting a generic lookup map, the template file locations
      and the waypoint path.*/
  PovrayOut( map<string, string*>& lookup, 
	     char* pov_tplfile,
	     char* ini_tplfile,
	     vector<Waypoint*> path);

  /** Kill event.*/
  virtual int delete_event_impl(GdkEventAny* event);

  /** update with a new waypoint path */
  void update_path(vector<Waypoint*> path) {
    ss_page.update_lookup( "PATH", get_povray_path(path));
  }

  /** update the endtime */
  void update_endtime(double time) {
    po_page.update_endtime( time);
  }
  
};



#endif
